#!/bin/sh

####################################################################
# (C) Copyright 2019 Hewlett Packard Enterprise Development LP.
###################################################################

############################################
#Function to trim the input and get the name
#Eg: Input: /sys/fs/cgroup/cpu,cpuacct/sg.slice
#    Output: sg.slice
############################################

#Exit 1: If value of sg.slice is between 0 and 950000
#Exit 2: If value of another slice is between 0 and 950000
#Exit 3: If value of another slice is 950000

function get_name(){
    echo "$1" | rev | cut -d'/' -f1 | rev 2>/dev/null
}

function rt_error() {
    slice=$(get_name "$1")
    if [ -n "$pid" ]; then
        PARENT_COMMAND=$(ps -o comm= $PPID)
        /bin/logger "ERROR: The cpu.rt_runtime_us value of $slice is $2. " -t $PARENT_COMMAND[$pid]
        /bin/logger "Serviceguard slice (sg.slice) needs the complete realtime quantum ($sched_rt_runtime_us) to operate " -t $PARENT_COMMAND[$pid]
        /bin/logger "and will be unable to do so if there is another slice with a realtime quantum." -t $PARENT_COMMAND[$pid]
        /bin/logger "For more information on how to resolve it please refer to the " -t $PARENT_COMMAND[$pid]
        /bin/logger "HPE Serviceguard for Linux documentation on the " -t $PARENT_COMMAND[$pid]
        /bin/logger "HPE Support Center at https://support.hpe.com/hpesc/home " -t $PARENT_COMMAND[$pid]
        /bin/logger "A Customer Advisory is published with additional information on this problem. " -t $PARENT_COMMAND[$pid]
    else
        echo "ERROR: The cpu.rt_runtime_us value of $slice is $2. " >&2
        echo "Serviceguard slice (sg.slice) needs the complete realtime quantum ($sched_rt_runtime_us) to operate " >&2
        echo "and will be unable to do so if there is another slice with a realtime quantum. " >&2
        echo "For more information on how to resolve it please refer to the " >&2
        echo "HPE Serviceguard for Linux documentation on the " >&2
        echo "HPE Support Center at https://support.hpe.com/hpesc/home " >&2
        echo "A Customer Advisory is published with additional information on this problem. " >&2

    fi
}

function check_realtime_value_of_slice() {
    file="$1/cpu.rt_runtime_us"
    realtime_value=`cat $file`
    echo "$realtime_value" 2>/dev/null
    if [[ "$realtime_value" != "$sched_rt_runtime_us" && "$realtime_value" != "0" ]];
    then
        if [[ "$1" != "$sg_slice" ]]; 
        then
            rt_error "$1" "$realtime_value"
        fi
        return 1
    fi
    return 0
}

function find_slice_with_rt() {
    #Store all slices except sg_slice and the default slices(system.slice, machine.slice and user.slice)
    slices=( `find /sys/fs/cgroup/cpu,cpuacct -maxdepth 1 ! -name "*system.slice" ! -name "*user.slice" ! -name "*machine.slice" ! -name "*$(get_name $sg_slice)" -name "*.slice"` )
    for i in "${slices[@]}";
        do
            rt_value=$(check_realtime_value_of_slice "$i")
            if [[ "$rt_value" == "$sched_rt_runtime_us" ]];
            then
                rt_error "$i" "$rt_value"
                return 3 #If value of another slice is 950000
            elif [[ "$rt_value" != "0" ]];
            then
                return 2
            fi
        done
    return 0
}

function create_sg_realtime_config_service() {
    if [[ ! -f "$1" ]];
    then
        slice=$(get_name $2)
        echo "[Unit]" > $1
        echo "Description=Serviceguard Real Time Slice configuration service" >> $1
        echo "BindsTo=$slice" >> $1
        echo "After=$slice" >> $1
        echo "" >> $1
        echo "[Service]" >> $1
        echo "Type=oneshot" >> $1
        echo "ExecStart=/bin/sh -c \"/usr/sbin/sysctl -n kernel.sched_rt_runtime_us > $2/cpu.rt_runtime_us\"" >> $1
        echo "RemainAfterExit=yes" >> $1
    fi
}

function create_sg_slice() {
    if [[ ! -f "$1" ]];
    then
        echo "Creating sg.slice ..."
        echo "[Unit]" > $1
        echo "Description=Serviceguard Real Time Slice" >> $1
        echo "Before=slices.target" >> $1
        echo "Requires=sg-realtime-config.service" >> $1
        echo "" >> $1
        echo "[Slice]" >> $1
        echo "CPUAccounting=yes" >> $1
        echo "" >> $1
        echo "[Install]" >> $1
        echo "WantedBy=slices.target" >> $1
    fi
}

function create_slice_conf() {
    if [[ ! -d "$1.d" ]];
    then
        mkdir -p $1.d
        slice=$(get_name $2)
        echo "[Service]" > $1.d/slice.conf
        echo "Slice=$slice" >> $1.d/slice.conf
    fi
}

function add_pid() {
    pid=$1
    slice=$2
    tasks=$slice/tasks
    echo $pid >> $tasks
    echo "$sched_rt_runtime_us" > $sg_slice/cpu.rt_runtime_us 2>/dev/null
}

#Start of main
while [[ $# -gt 0 ]]; do
      opt="$1"
      shift;
      current_arg="$1"
      case "$opt" in
        "-s") service="$1"; shift;;
        "-p") pid="$1"; shift;;
        "-i") install=true;
      esac
done

sg_slice_exists=false
sg_slice_eligible=false
sched_rt_runtime_us=`/usr/sbin/sysctl -n kernel.sched_rt_runtime_us`
sg_slice="/sys/fs/cgroup/cpu,cpuacct/sg.slice"
sg_realtime_config="/etc/systemd/system/sg-realtime-config.service"
sg_slice_location="/etc/systemd/system/sg.slice"
push_rt_quantum=false

if [[ "$install" = true ]]; then
    create_sg_slice "$sg_slice_location"
    create_slice_conf "$service" "$sg_slice"
    create_sg_realtime_config_service "$sg_realtime_config" "$sg_slice"
    exit 0
fi

if [ -d "$sg_slice" ];
then
    sg_slice_exists=true
    echo "$sched_rt_runtime_us" > $sg_slice/cpu.rt_runtime_us 2>/dev/null
    if [[ $? != "0" ]];
    then
        rt_value=$(check_realtime_value_of_slice "$sg_slice")
        if [[ $? != "0" ]];
        then
            push_rt_quantum=false
            if [ -n "$pid" ]; then
                PARENT_COMMAND=$(ps -o comm= $PPID)
                /bin/logger "ERROR: $(get_name "$sg_slice") does not have the complete quantum value." -t $PARENT_COMMAND[$pid]
                /bin/logger "Serviceguard needs complete quantum $sched_rt_runtime_us to operate." -t $PARENT_COMMAND[$pid]
                /bin/logger "For more information on how to resolve it please refer to the " -t $PARENT_COMMAND[$pid]
                /bin/logger "HPE Serviceguard for Linux documentation on the " -t $PARENT_COMMAND[$pid]
                /bin/logger "HPE Support Center at https://support.hpe.com/hpesc/home " -t $PARENT_COMMAND[$pid]
                /bin/logger "A Customer Advisory is published with additional information on this problem. " -t $PARENT_COMMAND[$pid]
            else
                echo "ERROR: $(get_name "$sg_slice") does not have the complete quantum value."
                echo "Serviceguard needs complete quantum $sched_rt_runtime_us to operate."
                echo "For more information on how to resolve it please refer to the " >&2
                echo "HPE Serviceguard for Linux documentation on the " >&2
                echo "HPE Support Center at https://support.hpe.com/hpesc/home " >&2
                echo "A Customer Advisory is published with additional information on this problem. " >&2
            fi
            exit 1
        fi
        if [[ "$rt_value" == "$sched_rt_runtime_us" ]];
        then
            sg_slice_eligible=true
        fi
    else
        push_rt_quantum=true
    fi
fi

#Check if another slice exists with value > 0
if [[ "$push_rt_quantum" = false && "$sg_slice_eligible" = false ]];
then
    find_slice_with_rt
    return_code=`echo $?`
    if [[ "$return_code" != "0" ]];
    then
        exit $return_code #Value of another slice is random
    else
        if [[ "$sg_slice_exists" = false ]];
        then
            create_sg_slice "$sg_slice_location"
        fi
    fi
fi

slice=$(get_name "$sg_slice")
#In case serviceguard is removed when qs is down, while installing SG
#the file sg.slice will also get deleted.
#So when qs.service tries to come up, the file will not exist.
#Hence, we will create the sg.slice file, if itsn't already present
create_sg_slice "$sg_slice_location"
if [ -n "$pid" ]; then
    systemctl start $slice
    add_pid "$pid" "$sg_slice"
else
    create_slice_conf "$service" "$sg_slice"
fi
create_sg_realtime_config_service "$sg_realtime_config" "$sg_slice"